@@ -647,7 +647,7 @@ start_moded_tabling(Closure, Wrapper, Worker, WrapperNoModes, ModeArgs) :-
                [Wrapper, ModeArgs, LStatus]),
         moded_done_leader(LStatus, Status, Skeleton, ModeArgs, Trie)
     ;   Status == invalid
-    ->  reeval(Trie),
+    ->  reeval(Trie, Wrapper, Skeleton),
         moded_gen_answer(Trie, Skeleton, ModeArgs)
     ;   % = run_follower, but never fresh and Status is a worklist
         shift(call_info(Skeleton/ModeArgs, Status))
@@ -1598,6 +1598,15 @@ mon_propagate(Action, Head) :-
         propagate_assert(Head),
         '$tbl_propagate_end'(Old)).
 mon_propagate(retract, Head) :-
+    !,
+    mon_abolish_dependents(Head).
+mon_propagate(rollback(Action), Head) :-
+    mon_propagate_rollback(Action, Head).
+
+mon_propagate_rollback(Action, _Head) :-
+    assert_action(Action),
+    !.
+mon_propagate_rollback(retract, Head) :-
     mon_abolish_dependents(Head).
 
 assert_action(asserta).
