@@ -181,7 +181,7 @@ Host A Process 2:
 All incomming trafic is handled  by  a   single  thread  with  the alias
 `udp_inbound_proxy`. This thread also performs  the internal dispatching
 using broadcast/1 and broadcast_request/1. Future   versions may provide
-for handling these requests in seperate threads.
+for handling these requests in separate threads.
 
 
 ## Caveats {#udp-broadcase-caveats}
@@ -352,12 +352,13 @@ udp_broadcast_address(IPAddress, Subnet, BroadcastAddress) :-
     udp_public_socket/4,                        % Scope, Port, Socket, FileNo
     udp_closed/1.				% Scope
 
-udp_inbound_proxy :-
+udp_inbound_proxy(Master) :-
     thread_at_exit(inbound_proxy_died),
+    make_private_socket,
+    thread_send_message(Master, udp_inbound_ready),
     udp_inbound_proxy_loop.
 
 udp_inbound_proxy_loop :-
-    make_private_socket,
     forall(udp_scope(Scope, ScopeData),
            make_public_socket(ScopeData, Scope)),
     retractall(udp_closed(_)),
@@ -522,6 +523,7 @@ ld_dispatch(S, request(Key, Term), From, Scope) :-
            safely((udp_term_string(Scope, reply(Key,Term), Message),
                    udp_send(S, Message, From, [])))).
 ld_dispatch(_S, send(Term), _From, _Scope) :-
+    !,
     safely_det(broadcast(Term)).
 ld_dispatch(_S, reply(Key, Term), From, _Scope) :-
     (   reply_queue(Key, Queue)
@@ -570,10 +572,12 @@ reload_inbound_proxy :-
           fail),
     !.
 reload_inbound_proxy :-
-    thread_create(udp_inbound_proxy, _,
+    thread_self(Me),
+    thread_create(udp_inbound_proxy(Me), _,
                   [ alias(udp_inbound_proxy),
                     detached(true)
-                  ]).
+                  ]),
+    thread_get_message(Me, udp_inbound_ready, [timeout(10)]).
 
 inbound_proxy_died :-
     thread_self(Self),
