@@ -38,6 +38,7 @@
 #include "pl-transaction.h"
 #include "pl-event.h"
 #include "pl-dbref.h"
+#include "pl-tabling.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This module implements _transactions_, notably _isolating_ transactions.
@@ -85,11 +86,13 @@ A clause is visible iff
 #define IS_ASSERT_GEN(g) ((g)==GEN_ASSERTA||(g)==GEN_ASSERTZ)
 
 typedef struct tr_stack
-{ struct tr_stack *parent;		/* parent transaction */
-  gen_t		   gen_nest;		/* Saved nesting generation */
-  gen_t            generation;		/* Parent generation */
-  Table		   clauses;		/* Parent changed clauses */
-  term_t	   id;			/* Parent goal */
+{ struct tr_stack  *parent;		/* parent transaction */
+  gen_t		    gen_nest;		/* Saved nesting generation */
+  gen_t             generation;		/* Parent generation */
+  Table		    clauses;		/* Parent changed clauses */
+  struct tbl_trail *table_trail;	/* Parent changes to tables */
+  term_t	    id;			/* Parent goal */
+  unsigned int	    flags;		/* TR_* flags */
 } tr_stack;
 
 static void
@@ -290,7 +293,8 @@ transaction_discard(ARG1_LD)
 		  action = ATOM_retract;
 		}
 
-		if ( def && def->events && action )
+		if ( def && def->events && action &&
+		     !(LD->transaction.flags&TR_BULK) )
 		{ if ( !predicate_update_event(def, action, cl,
 					       P_EVENT_ROLLBACK PASS_LD) )
 		    rc = FALSE;
@@ -369,6 +373,24 @@ transaction_updates(Buffer b ARG_LD)
 	  cmp_updates);
   }
 
+  return TRUE;
+}
+
+
+static int
+announce_updates(Buffer updates ARG_LD)
+{ tr_update *u, *e;
+
+  u = baseBuffer(updates, tr_update);
+  e = topBuffer(updates, tr_update);
+
+  for(; u<e; u++)
+  { Definition def = u->clause->predicate;
+
+    if ( !predicate_update_event(def, nameFunctor(u->update), u->clause, 0 PASS_LD) )
+      return FALSE;
+  }
+
   return TRUE;
 }
 
@@ -377,45 +399,62 @@ transaction_updates(Buffer b ARG_LD)
 		 *	PROLOG CONNECTION	*
 		 *******************************/
 
-#define TR_TRANSACTION		0x0001
-#define TR_SNAPSHOT		0x0002
-
 static int
 transaction(term_t goal, term_t constraint, term_t lock, int flags ARG_LD)
 { int rc;
+  buffer updates;
+
+  initBuffer(&updates);
 
   if ( LD->transaction.generation )
-  { tr_stack parent = { .generation = LD->transaction.generation,
-			.gen_nest   = LD->transaction.gen_nest,
-			.clauses    = LD->transaction.clauses,
-			.parent     = LD->transaction.stack,
-			.id         = LD->transaction.id
+  { tr_stack parent = { .generation  = LD->transaction.generation,
+			.gen_nest    = LD->transaction.gen_nest,
+			.clauses     = LD->transaction.clauses,
+			.table_trail = LD->transaction.table_trail,
+			.parent      = LD->transaction.stack,
+			.id          = LD->transaction.id,
+			.flags       = LD->transaction.flags
 		      };
 
     LD->transaction.clauses  = NULL;
     LD->transaction.id       = goal;
     LD->transaction.stack    = &parent;
     LD->transaction.gen_nest = LD->transaction.generation;
+    LD->transaction.flags    = flags;
     rc = callProlog(NULL, goal, PL_Q_PASS_EXCEPTION, NULL);
     if ( rc && constraint )
       rc = callProlog(NULL, constraint, PL_Q_PASS_EXCEPTION, NULL);
     if ( rc && (flags&TR_TRANSACTION) )
     { if ( LD->transaction.clauses )
       { if ( parent.clauses )
-	{ merge_tables(parent.clauses, LD->transaction.clauses);
+	{ if ( (flags&TR_BULK) )
+	  { transaction_updates(&updates PASS_LD);
+	    if ( !announce_updates(&updates PASS_LD) )
+	      goto nested_discard;
+	  }
+	  merge_tables(parent.clauses, LD->transaction.clauses);
 	  destroyHTable(LD->transaction.clauses);
 	} else
 	{ parent.clauses = LD->transaction.clauses;
 	}
+	if ( parent.table_trail )
+	{ merge_tabling_trail(parent.table_trail, LD->transaction.table_trail);
+	} else
+	{ parent.table_trail = LD->transaction.table_trail;
+	}
       }
     } else
-    { rc = transaction_discard(PASS_LD1) && rc;
+    { nested_discard:
+      rc = transaction_discard(PASS_LD1) && rc;
+      rc = transaction_rollback_tables(PASS_LD1) && rc;
       LD->transaction.generation = parent.generation;
     }
-    LD->transaction.gen_nest = parent.gen_nest;
-    LD->transaction.clauses  = parent.clauses;
-    LD->transaction.stack    = parent.parent;
-    LD->transaction.id       = parent.id;
+    LD->transaction.gen_nest    = parent.gen_nest;
+    LD->transaction.clauses     = parent.clauses;
+    LD->transaction.table_trail = parent.table_trail;
+    LD->transaction.stack       = parent.parent;
+    LD->transaction.id          = parent.id;
+    LD->transaction.flags	= parent.flags;
   } else
   { int tid = PL_thread_self();
 #ifdef O_PLMT
@@ -441,15 +480,22 @@ transaction(term_t goal, term_t constraint, term_t lock, int flags ARG_LD)
 	LD->transaction.gen_start = global_generation();
 	rc = callProlog(NULL, constraint, PL_Q_PASS_EXCEPTION, NULL);
       }
+      if ( rc && (flags&TR_BULK) )
+      { transaction_updates(&updates PASS_LD);
+	rc = announce_updates(&updates PASS_LD);
+      }
       if ( rc )
-      { rc = transaction_commit(PASS_LD1);
+      { rc = ( transaction_commit_tables(PASS_LD1) &&
+	       transaction_commit(PASS_LD1) );
 	if ( constraint ) TR_UNLOCK();
       } else
       { if ( constraint ) TR_UNLOCK();
 	transaction_discard(PASS_LD1);
+	transaction_rollback_tables(PASS_LD1);
       }
     } else
     { rc = transaction_discard(PASS_LD1) && rc;
+      rc = transaction_rollback_tables(PASS_LD1) && rc;
     }
     LD->transaction.id         = 0;
     LD->transaction.generation = 0;
@@ -458,12 +504,29 @@ transaction(term_t goal, term_t constraint, term_t lock, int flags ARG_LD)
     LD->transaction.gen_start  = 0;
   }
 
+  if ( (flags&TR_BULK) )
+    discardBuffer(&updates);
+
   return rc;
 }
 
+static const opt_spec transaction_options[] =
+{ { ATOM_bulk,		 OPT_BOOL },
+  { NULL_ATOM,		 0 }
+};
+
 static
-PRED_IMPL("$transaction", 1, transaction, PL_FA_TRANSPARENT)
+PRED_IMPL("$transaction", 2, transaction, PL_FA_TRANSPARENT)
 { PRED_LD
+  int flags = TR_TRANSACTION;
+  int bulk = FALSE;
+
+  if ( !scan_options(A2, 0,
+		     ATOM_transaction_option, transaction_options,
+		     &bulk) )
+    return FALSE;
+  if ( bulk )
+    flags |= TR_BULK;
 
   return transaction(A1, 0, 0, TR_TRANSACTION PASS_LD);
 }
@@ -624,7 +687,7 @@ PRED_IMPL("pred_generations", 1, pred_generations, PL_FA_TRANSPARENT)
 #define NDET PL_FA_NONDETERMINISTIC
 
 BeginPredDefs(transaction)
-  PRED_DEF("$transaction",        1, transaction,         META)
+  PRED_DEF("$transaction",        2, transaction,         META)
   PRED_DEF("$transaction",        3, transaction,         META)
   PRED_DEF("$snapshot",           1, snapshot,            META)
   PRED_DEF("current_transaction", 1, current_transaction, NDET|META)
