@@ -100,6 +100,7 @@ typedef struct trie_node
 #define TRIE_ISSHARED	0x0004		/* This is a shared answer trie */
 #define TRIE_COMPLETE	0x0008		/* Answer trie is complete */
 #define TRIE_ABOLISH_ON_COMPLETE 0x0010	/* Abolish the table when completed */
+#define TRIE_ISTRACKED  0x0020		/* Trie changes are tracked */
 
 typedef struct trie
 { atom_t		symbol;		/* The associated symbol */
@@ -133,6 +134,11 @@ typedef struct trie
   } data;
 } trie;
 
+typedef struct size_abstract
+{ int		from_depth;		/* start below depth */
+  size_t	size;			/* limit each term to size */
+} size_abstract;
+
 #define acquire_trie(t) ATOMIC_INC(&(t)->references)
 #define release_trie(t) do { if ( ATOMIC_DEC(&(t)->references) == 0 ) \
 			       trie_clean(t); \
@@ -147,6 +153,11 @@ typedef struct trie
 #define TRIE_STAT_INC(t, v) ((void)0)
 #endif
 
+/* trie_lookup_abstract() return values (< 0: error) */
+#define TRIE_ABSTRACTED			  2
+#define TRIE_LOOKUP_CONTAINS_ATTVAR	-10
+#define TRIE_LOOKUP_CYCLIC		-11
+
 COMMON(void)	initTries(void);
 COMMON(trie *)	trie_create(alloc_pool *pool);
 COMMON(void)	trie_destroy(trie *trie);
@@ -162,8 +173,10 @@ COMMON(int)	get_trie(term_t t, trie **tp);
 COMMON(int)	get_trie_noex(term_t t, trie **tp);
 COMMON(int)	unify_trie_term(trie_node *node, trie_node **parent,
 				term_t term ARG_LD);
-COMMON(int)	trie_lookup(trie *trie, trie_node *root, trie_node **nodep,
-			    Word k, int add, TmpBuffer vars ARG_LD);
+COMMON(int)	trie_lookup_abstract(trie *trie,
+				     trie_node *root, trie_node **nodep, Word k,
+				     int add, size_abstract *abstract,
+				     TmpBuffer vars ARG_LD);
 COMMON(int)	trie_error(int rc, term_t culprit);
 COMMON(int)	trie_trie_error(int rc, trie *trie);
 COMMON(atom_t)	trie_symbol(trie *trie);
@@ -185,4 +198,11 @@ COMMON(void *)	map_trie_node(trie_node *n,
 			      void* (*map)(trie_node *n, void *ctx), void *ctx);
 COMMON(atom_t)	compile_trie(Definition def, trie *trie ARG_LD);
 
+static inline int
+trie_lookup(trie *trie, trie_node *node, trie_node **nodep,
+	    Word k, int add, TmpBuffer vars ARG_LD)
+{ return trie_lookup_abstract(trie, node, nodep, k, add,
+			      NULL, vars PASS_LD);
+}
+
 #endif /*_PL_TRIE_H*/
